	#define half	float
	#define half2	vec2
	#define half3	vec3
	#define half4	vec4
	#define float2	vec2
	#define float3	vec3
	#define float4	vec4

	#define MulMatrix(Mtx, Vect)	(Mtx * Vect)
	#define saturate(x)				clamp(x, 0.0, 1.0)
	#define frac(x)					fract(x)
	#define lerp( A, B, Factor )	mix( A, B, Factor )

	#define texture2DBias(Texture,UV,Bias)				texture2D(Texture, UV, Bias)
	#if USE_GL_EXT_shader_texture_lod
		#define texture2DLod(Texture,UV,Lod)			texture2DLodEXT(Texture, UV.xy, Lod)
	#else
		#define texture2DLod(Texture,UV,Lod)			texture2D(Texture, UV.xy)
	#endif

	#define tex2Dlod(Texture,UVLod)						texture2DLod(Texture, UVLod.xy, UVLod.w )
	#define texDepth2Dlod(Texture,UVLod)				texture2DLod(Texture, UVLod.xy, UVLod.w )

	#define OUT_VARYING_HIGH(Type,Name,Semantics)		varying highp Type Name
	#define OUT_VARYING_MEDIUM(Type,Name,Semantics)		varying mediump Type Name
	#define OUT_VARYING_LOW(Type,Name,Semantics)		varying lowp Type Name
	#define OUT_VARYING_DEFAULT(Type,Name,Semantics)	OUT_VARYING_MEDIUM(Type,Name,Semantics)
	#define OUT_VARYING_ARRAY_DEFAULT(Type,Name,Subs,Semantics)	OUT_VARYING_DEFAULT(Type,Name[Subs],Semantics)

	#define IN_VARYING_HIGH(Type,Name,Semantics)		OUT_VARYING_HIGH(Type,Name,Semantics)
	#define IN_VARYING_MEDIUM(Type,Name,Semantics)		OUT_VARYING_MEDIUM(Type,Name,Semantics)
	#define IN_VARYING_LOW(Type,Name,Semantics)			OUT_VARYING_LOW(Type,Name,Semantics)
	#define IN_VARYING_DEFAULT(Type,Name,Semantics)		OUT_VARYING_DEFAULT(Type,Name,Semantics)
	#define IN_VARYING_ARRAY_DEFAULT(Type,Name,Subs,Semantics)	IN_VARYING_DEFAULT(Type,Name[Subs],Semantics)

	// These statements MUST not be followed by semi-colons!
	#define IN_BUILTIN(Type,Name,Semantics)
	#define OUT_BUILTIN(Type,Name,Semantics)

	#define UNIFORM(Type,Name)							uniform Type Name
	#define UNIFORM_SAMPLER2D(Name,Semantics)			UNIFORM(sampler2D,Name)
	#define UNIFORM_HIGH(Type,Name)						UNIFORM(highp Type,Name)
	#define UNIFORM_MEDIUM(Type,Name)					UNIFORM(mediump Type,Name)
	#define UNIFORM_LOW(Type,Name)						UNIFORM(lowp Type,Name)

	#define ATTRIBUTE(Type,Name,Semantics)				attribute Type Name

UNIFORM(vec4, SceneCoordinate1ScaleBias);
UNIFORM(vec4, SceneCoordinate2ScaleBias);
UNIFORM(vec4, SceneCoordinate3ScaleBias);

ATTRIBUTE(vec4, Position, POSITION);
ATTRIBUTE(vec2, TexCoords0, TEXCOORD0);

OUT_VARYING_DEFAULT(vec4, ScreenPosition, TEXCOORD0);
OUT_VARYING_DEFAULT(vec4, FilterBufferUV, TEXCOORD1);
OUT_VARYING_DEFAULT(vec2, PixelPos, TEXCOORD2);

void main()
{
	// Target position
	gl_Position = Position;								
	
	// Background position
	ScreenPosition.xy = Position.xy;
	// full resolution
	FilterBufferUV.xy = TexCoords0;

	// compute pixel position from screen pos, unify D3D9 half texel shift (done like this for cleaner code), flip y
	vec2 PixelPos = Position.xy / Position.w * SceneCoordinate1ScaleBias.xy + SceneCoordinate1ScaleBias.wz;

	PixelPos = PixelPos;

	// half resolution
	FilterBufferUV.zw = PixelPos * SceneCoordinate2ScaleBias.xy + SceneCoordinate2ScaleBias.wz;
	// quarter resolution
	ScreenPosition.zw = PixelPos * SceneCoordinate3ScaleBias.xy + SceneCoordinate3ScaleBias.wz;
}
