uniform highp sampler2D  SceneColor;
uniform lowp vec2  BloomScaleAndThreshold;
uniform highp vec2 g_avSampleOffsets[16];

varying mediump vec2  TexCoord0;

highp vec3 DecodeRGBE8( highp vec4 rgbe )
{
	highp vec3 vDecoded;

    // Retrieve the shared exponent
	highp float fExp = rgbe.a * 255.0 - 128.0;
	
	// Multiply through the color components
	vDecoded = rgbe.rgb * exp2(fExp);
	
	return vDecoded;
}

mediump vec3 DecodeRGB16( mediump vec3 rgb )
{
	return rgb * 2.0;
}

void main()
{
    highp vec3 vColor = vec3( 0.0, 0.0, 0.0 );
    
    for( int i = 0; i < 9; i++ )
    {
        // Compute the sum of color values
        highp vec4 vSample = texture2D( SceneColor, TexCoord0 + g_avSampleOffsets[i] );
        
        vColor += vSample.rgb;
    }
    
    // Divide the sum to complete the average
    vColor /= 9.0;
 
    // Bright pass and tone mapping
    vColor *= vColor - BloomScaleAndThreshold.y;
   	
    gl_FragColor = vec4(vColor * BloomScaleAndThreshold.x, 1.0);
}