uniform sampler2D  sBaseTex;
uniform sampler2D  sNormalMap;
uniform sampler2D  sSpecularMap;

uniform lowp float  LightIntensity;
uniform lowp vec3  LightColor;

varying lowp    vec3  TangentSpaceLightDir;
varying mediump vec2  TexCoord;

varying lowp vec3  TangentSpaceEyeToVertex;

mediump vec4 EncodeRGBE8( mediump vec3 rgb )
{
	mediump vec4 vEncoded;

    // Determine the largest color component
	mediump float maxComponent = max( max(rgb.r, rgb.g), rgb.b );
	
	// Round to the nearest integer exponent
	mediump float fExp = ceil( log2(maxComponent) );

    // Divide the components by the shared exponent
	vEncoded.rgb = rgb / exp2(fExp);
	
	// Store the shared exponent in the alpha channel
	vEncoded.a = (fExp + 128.0) / 255.0;

	return vEncoded;
}

mediump vec3 EncodeRGB16( mediump vec3 rgb )
{
	return rgb / 2.0;
}

void main()
{
	lowp vec3 normal = ( texture2D(sNormalMap, TexCoord).rgb * 2.0 - 1.0 );
	
	lowp float light = max( dot(TangentSpaceLightDir, normal) * LightIntensity, 0.0 );

	lowp vec4 texColor = texture2D(sBaseTex, TexCoord);//vec4(0.7, 0.7, 0.7, 1.0);//	

	lowp vec3 cSpecular = texture2D( sSpecularMap, TexCoord ).rgb;//vec3(1.0, 1.0, 1.0);//

	// tangent space specular
    lowp vec3 TangentSpaceReflectionVector = reflect( TangentSpaceLightDir, normal );
    lowp float BaseSpec = max( 0.0, dot( TangentSpaceEyeToVertex, TangentSpaceReflectionVector ) );   // VDotR

	lowp float SpecularAmount = pow( BaseSpec, 10.0 );

	texColor.rgb *= light * LightColor;
	texColor.rgb += SpecularAmount*cSpecular;

	texColor.rgb = EncodeRGB16( texColor.rgb );

	gl_FragColor = texColor;
}
