
attribute highp vec4	Position;

attribute highp vec4	Normal;
attribute highp vec3	Tangent;
attribute mediump float BlendWeight;

attribute highp vec2	UV;
attribute highp vec4	Color;

uniform highp mat4	WVPMat;
uniform highp vec3	CameraRight;
uniform highp vec3	CameraUp;
uniform highp vec3	EyePos;

uniform mediump float	ParticleScreenAlignment;

uniform mediump mat4 SpriteCorner;

varying lowp vec4		VertexColor;
varying mediump vec2	TexCoord;

void main()
{
	int nCornerId = int(Position.w);
	int iScreenAlignment = int(ParticleScreenAlignment);

	highp vec3 right = CameraRight;
	highp vec3 up = CameraUp;

	vec3 size = Tangent;

	highp vec4 pos = Position;

	highp float Rotate = BlendWeight;

	if( iScreenAlignment == 0 )
	{
		size.y = Tangent.x;
	}
	else if ( iScreenAlignment == 2 )
	{
		up = normalize( Position.xyz - Normal.xyz );
		highp vec3 vDir = EyePos - Position.xyz;
		right = normalize( cross( up, vDir ) );
		Rotate = 0.0;
	}
	else if ( iScreenAlignment == 3 )
	{
		up = vec3(0.0,0.0,-1.0);
		right = vec3(1.0,0.0,0.0);
		size.y = size.z;
	}

	if( Rotate != 0.0 )
	{
		float fs = cos( -Rotate );
		float fc = sin( -Rotate );

		highp vec3 orgRight = right;

		right = fs * orgRight + fc * up;
		up = -fc * orgRight + fs * up;
	}

	pos.xyz += SpriteCorner[nCornerId].x * right * size.x + SpriteCorner[nCornerId].y * up * size.y;
	pos.w = 1.0;

	gl_Position = WVPMat * pos;

	TexCoord = UV;

	VertexColor = Color;
}
