attribute highp vec4  Position;
attribute highp vec2  UV;
attribute highp vec4  Tangent;
attribute highp vec4  Normal;
attribute mediump vec4 BlendIndices;
attribute mediump vec4 BlendWeight;

uniform mediump mat4  World;
uniform mediump mat4  VPMat;		// model view projection transformation
uniform mediump vec3  LightPos;	// Light position (point light) in model space
uniform mediump vec3  LightDir;	// Light position (point light) in model space
uniform mediump vec3  EyePos;

uniform highp   mat4  BoneMatrices[60];

varying lowp vec3  TangentSpaceLightDir;
varying mediump vec2  TexCoord;

varying lowp vec3  TangentSpaceEyeToVertex;

vec3 TransformVector( vec3 inVector, ivec4 inBoneIndices )
{
	mediump vec3 o = ( vec4(inVector,0) * BoneMatrices[inBoneIndices.x] ).xyz * BlendWeight.x;
	o += ( vec4(inVector,0) * BoneMatrices[inBoneIndices.y] ).xyz * BlendWeight.y;
	o += ( vec4(inVector,0) * BoneMatrices[inBoneIndices.z] ).xyz * BlendWeight.z;
	o += ( vec4(inVector,0) * BoneMatrices[inBoneIndices.w] ).xyz * BlendWeight.w;
	return o;
}

vec4 TransformPoint( vec4 inPoint, ivec4 inBoneIndices )
{
	mediump vec4 o = ( inPoint * BoneMatrices[inBoneIndices.x] ) * BlendWeight.x;
	o += ( inPoint * BoneMatrices[inBoneIndices.y] ) * BlendWeight.y;
	o += ( inPoint * BoneMatrices[inBoneIndices.z] ) * BlendWeight.z;
	o += ( inPoint * BoneMatrices[inBoneIndices.w] ) * BlendWeight.w;
	return o;
}

#define DecodeComponent( Val )	( ( Val / 127.5 ) - 1.0 )
#define DecodeVector( Vec )		( ( Vec.xyz / vec3( 127.5 ) ) - vec3( 1.0 ) )

#define MatrixComponentCompose( idx ) BoneMatrices[boneIndex.x][idx] * BlendWeight.x + BoneMatrices[boneIndex.y][idx] * BlendWeight.y + BoneMatrices[boneIndex.z][idx] * BlendWeight.z/* + vec4(0.0,0.0,0.0,1.0) * BlendWeight.w*/

void main()
{
	mediump ivec4 boneIndex = ivec4(BlendIndices);

	/*mediump mat4 WorldMatrix = BoneMatrices[boneIndex.x] * BlendWeight.x
							 + BoneMatrices[boneIndex.y] * BlendWeight.y
							 + BoneMatrices[boneIndex.z] * BlendWeight.z
							 + BoneMatrices[boneIndex.w] * BlendWeight.w;*/

	mediump vec4 WorldMatrix[4];

	WorldMatrix[0] = MatrixComponentCompose( 0 );
	WorldMatrix[1] = MatrixComponentCompose( 1 );
	WorldMatrix[2] = MatrixComponentCompose( 2 );
	WorldMatrix[3] = MatrixComponentCompose( 3 );

	// Ҫתãֱ˷Ϊ任
	mediump vec4  WorldPosition = vec4( dot( Position, WorldMatrix[0] ),
										dot( Position, WorldMatrix[1] ),
										dot( Position, WorldMatrix[2] ),
										dot( Position, WorldMatrix[3] ) );

	// һ
	//WorldPosition /= WorldPosition.w;

	//highp vec3 lightDirection = normalize(LightPos - vec3(Position));

	mediump mat3 WorldRotate = mat3( WorldMatrix[0].xyz, WorldMatrix[1].xyz, WorldMatrix[2].xyz );

	//  Tangent space 任ռ䡣
	mediump vec3 TangentX = DecodeVector(Tangent);
	mediump vec3 TangentZ = DecodeVector(Normal);
	mediump vec3 TangentY = cross( TangentZ, TangentX ) * DecodeComponent( Normal.w );

	mediump vec3 WorldT = TangentX * WorldRotate;
	mediump vec3 WorldB = TangentY * WorldRotate;
	mediump vec3 WorldN = TangentZ * WorldRotate;

	/*
	mediump vec4  WorldPosition = TransformPoint( Position, boneIndex );
	mediump vec3  WorldT = TransformVector( Tangent, boneIndex );
	mediump vec3  WorldB = TransformVector( Binormal, boneIndex );
	mediump vec3  WorldN = TransformVector( Normal, boneIndex );
	*/

	// ռ䵽 Tangent Space ı任
	mediump mat3 WorldToTangent = mat3( WorldT, WorldB, WorldN );

	// Ϊ WorldToTangent תãֱ˷Ϊ任
	TangentSpaceLightDir = normalize( LightDir * WorldToTangent );

    mediump vec3 EyeToVertex = normalize( WorldPosition.xyz/WorldPosition.w - EyePos );

	TangentSpaceEyeToVertex = normalize( EyeToVertex * WorldToTangent );

	gl_Position = VPMat * WorldPosition;

	TexCoord = UV;
}
