attribute highp vec4  Position;
attribute highp vec2  UV;
attribute highp vec4  Tangent;
attribute highp vec4  Normal;

uniform mediump mat4  World;
uniform mediump mat3  WorldRotate;
uniform mediump mat4  WVPMat;		// model view projection transformation
uniform mediump vec3  LightPos;	// Light position (point light) in model space
uniform mediump vec3  LightDir;	// Light position (point light) in model space
uniform mediump vec3  EyePos;

varying lowp vec3  TangentSpaceLightDir;
varying mediump vec2  TexCoord;
varying mediump float VertexLighting;

varying lowp vec3  TangentSpaceEyeToVertex;

#define DecodeComponent( Val )	( ( Val / 127.5 ) - 1.0 )
#define DecodeVector( Vec )		( ( Vec.xyz / vec3( 127.5 ) ) - vec3( 1.0 ) )

void main()
{
	gl_Position = WVPMat * Position;
	
	//  Tangent space 任ռ䡣
	mediump vec3 TangentX = DecodeVector(Tangent);
	mediump vec3 TangentZ = DecodeVector(Normal);
	mediump vec3 TangentY = cross( TangentZ, TangentX ) * DecodeComponent( Normal.w );

	highp mat3 tangentSpaceXform = mat3(TangentX, TangentY, TangentZ);

	mediump mat3 WorldToTangent = mat3(World) * tangentSpaceXform;

	TangentSpaceLightDir = LightDir * WorldToTangent;

	mediump vec3  WorldPosition = vec3( World * Position );

    mediump vec3 EyeToVertex = normalize( WorldPosition - EyePos );

	TangentSpaceEyeToVertex = EyeToVertex * WorldToTangent;

	TexCoord = UV;
}
