attribute highp vec4  Position;
attribute highp vec2  UV;
attribute highp vec4  Color;
attribute mediump vec4 BlendIndices;
attribute mediump vec4 BlendWeight;

uniform highp mat4  VPMat;		// model view projection transformation
uniform mediump vec4  MaterialColor;

uniform highp   mat4  BoneMatrices[60];

varying mediump vec2  TexCoord;
varying lowp vec4  VertexColor;

vec4 MulBone4( vec4 vInputPos, int nMatrix, float fBlendWeight )
{
    vec4 vResult;
      
    vResult.x = dot( vInputPos, BoneMatrices[nMatrix][0] );
    vResult.y = dot( vInputPos, BoneMatrices[nMatrix][1] );
    vResult.z = dot( vInputPos, BoneMatrices[nMatrix][2] );
    vResult.w = dot( vInputPos, BoneMatrices[nMatrix][3] );
    
    return vResult * fBlendWeight;
}

void main()
{
	mediump ivec4 boneIndex = ivec4(BlendIndices);

	vec4 WorldPosition = MulBone4( Position, boneIndex.x, BlendWeight.x );
	WorldPosition += MulBone4( Position, boneIndex.y, BlendWeight.y );
	WorldPosition += MulBone4( Position, boneIndex.z, BlendWeight.z );

	gl_Position = VPMat * WorldPosition;

	VertexColor = Color * MaterialColor;

	TexCoord = UV;
}
